local HonorTracker = select(2, ...)

local Tracking = HonorTracker:RegisterModule("Tracking")
Tracking.serverMapping = {}

local Stats = HonorTracker:GetModule("Stats")
local L = HonorTracker.L

function Tracking:ConvertNameToQualified(name, rank)
	if( not self.serverMapping or not rank ) then return name end

	local fullName = self.serverMapping[name .. rank]
	return fullName or name
end

function Tracking:CHAT_MSG_COMBAT_HONOR_GAIN(text, playerName, languageName, channelName, playerName2, specialFlags, zoneChannelID, chanenlIndex, channelBaseName, unused, lineID, guid, bnSenderID, isMobile, isSubtitel, hideSenderInLetterbox, suppressRaidIcons)
	local directHonor = tonumber(string.match(text, L["been awarded (%d+) honor points"]))
	if( directHonor and directHonor > 0 ) then
		self.db.today.honor.objectives = (self.db.today.honor.objectives or 0) + directHonor
		self.db.today.honor.total = (self.db.today.honor.total or 0) + directHonor

		if( self.activeBattlefield ) then
			self.activeBattlefield.honor.total = self.activeBattlefield.honor.total + directHonor
			self.activeBattlefield.honor.objectives = self.activeBattlefield.honor.objectives + directHonor
		end

		HonorTracker:Trigger("OnHonor")
		return
	end

	local name, _other = string.split(" ", text, 2)
	name = self:ConvertNameToQualified(name, string.match(text, L["honorable kill Rank: (.+) %("]))

	self.db.today.kills[name] = self.db.today.kills[name] or {kills = 0, honor = 0}
	self.db.today.kills[name].kills = self.db.today.kills[name].kills + 1

	local honor = tonumber(string.match(text, L["Estimated Honor Points: (%d+)"]))
	local actualHonor = Stats:CalculateModifiedHonor(name, honor)

	self.db.today.honor.total = (self.db.today.honor.total or 0) + actualHonor
	self.db.today.kills[name].honor = self.db.today.kills[name].honor + actualHonor

	if( self.activeBattlefield ) then
		self.activeBattlefield.honor.total = self.activeBattlefield.honor.total + actualHonor
	end

	HonorTracker:Trigger("OnHonor", name)
end

function Tracking:CheckDayChange()
	local stateTags = {
		yesterday = string.join(", ", GetPVPYesterdayStats()),
		thisWeek = string.join(", ", GetPVPThisWeekStats()),
		lastWeek = string.join(", ", GetPVPLastWeekStats()),
	}

	-- Check for daily reset
	local yesterdayHonor = select(3, GetPVPYesterdayStats())
	if( self.db.stateTags.yesterday and self.db.stateTags.yesterday ~= stateTags.yesterday ) then
		-- Record our daily stats
		local stats = Stats:CalculateToday()
		stats.time = self.db.today.start
		stats.honor = CopyTable(self.db.today)
		stats.honor.actual = yesterdayHonor
		table.insert(self.db.thisWeek, stats)

		-- Reset our stats for today
		self.db.yesterday = CopyTable(self.db.today)
		self.db.yesterday.stats = Stats:CalculateToday()
		self.db.today = {}

		HonorTracker:CheckDB()
		HonorTracker:Trigger("OnDailyReset")
	end

	-- Check for weekly reset
	if( self.db.stateTags.lastWeek and self.db.stateTags.lastWeek ~= stateTags.lastWeek ) then
		self.db.lastWeek = CopyTable(self.db.thisWeek)
		self.db.thisWeek = {}

		self.db.rankPoints.lastWeek = self.db.rankPoints.thisWeek
		self.db.rankPoints.thisWeek = Stats:CalculateRankPoints()

		-- After one weekly reset clear out our old data
		self.db.trackingOld = nil

		HonorTracker:CheckDB()
		HonorTracker:Trigger("OnWeeklyReset")
	end

	self.db.stateTags = stateTags
end

function Tracking:CacheServerMapping()
	if( not InActiveBattlefield() ) then
		self.serverMapping = {}
		return
	end

	-- 1 is Alliance and 0 is Horde
	local otherFaction
	if( UnitFactionGroup("player") == "Horde" ) then
		otherFaction = 1
	else
		otherFaction = 0
	end

	self.serverMapping = {}
	for i=1, GetNumBattlefieldScores() do
		local name, _, _, _, _, faction, rank = GetBattlefieldScore(i);
		if( faction == otherFaction ) then
			local nameWithoutServer, server = string.split("-", name, 2)
			if( server and server ~= "" ) then
				-- Everyone without a rank defaults to #5 as far as the honorable kill message is concerned
				local rankName = GetPVPRankInfo(rank, faction) or GetPVPRankInfo(5, faction)
				self.serverMapping[nameWithoutServer .. rankName] = name
			end
		end
	end
end

function Tracking:UPDATE_BATTLEFIELD_STATUS()
	local hasActive = false
	for i=1, MAX_BATTLEFIELD_QUEUES do
		local status, map, instanceID, isRegistered, suspendedQueue, queueType, gameType, role = GetBattlefieldStatus(i)
		if( status == "active" ) then
			hasActive = true

			self.activeBattlefieldMap = map
			self.activeBattlefieldID = instanceID
		end
	end

	-- Active battlefield but no active one created yet
	if( not self.activeBattlefield and self.activeBattlefieldID ) then
		-- Find an active battlefield to restore data for
		for _, data in pairs(self.db.today.battlegrounds) do
			if( not data.finished ) then
				-- Found an existing one
				if( data.map == self.activeBattlefieldMap and data.instanceID == self.activeBattlefieldID ) then
					self.activeBattlefield = data
				-- Bugged or we AFKed out. We have data, but no idea what happened
				elseif( data.instanceID ) then
					data.instanceID = nil
					data.finished = true
					data.partial = true
				end
			end
		end

		-- Didn't find an existing one to restore
		if( not self.activeBattlefield ) then
			self.activeBattlefield = {
				map = self.activeBattlefieldMap,
				instanceID = self.activeBattlefieldID,
				honor = {
					total = 0,
					objectives = 0
				}
			}

			table.insert(self.db.today.battlegrounds, self.activeBattlefield)
		end
	end
	
	if( self.activeBattlefield and self.activeBattlefield.runtime and not hasActive ) then
		-- We left by getting AFK'd out
		if( not self.activeBattlefield.winner ) then
			self.activeBattlefield.instanceID = nil
			self.activeBattlefield.partial = true
			self.activeBattlefield.finished = true
		end

		HonorTracker:Trigger("OnBattlefieldFinished", self.activeBattlefield)
		self.activeBattlefield = nil
	end

	-- We're done with our battlefield
	if( not hasActive ) then
		self.activeBattlefieldMap = nil
		self.activeBattlefieldID = nil
	end
end

function Tracking:PLAYER_ENTERING_WORLD()
	HonorTracker:CheckDB()

	self:CacheServerMapping()
	self:CheckDayChange()
end

function Tracking:UPDATE_BATTLEFIELD_SCORE()
	self:CacheServerMapping()

	-- Ordering is a bit weird here, and we actually update runtime/winner here
	-- and then output once we get kicked out based on queue state
	if( self.activeBattlefield and GetBattlefieldInstanceRunTime() ) then
		self.activeBattlefield.runtime = math.floor(GetBattlefieldInstanceRunTime() / 1000)
			
		if( GetBattlefieldWinner() ) then
			self.activeBattlefield.instanceID = nil
			self.activeBattlefield.finished = true

			if( GetBattlefieldWinner() == 0 ) then
				self.activeBattlefield.winner = "Horde"
			else
				self.activeBattlefield.winner = "Alliance"
			end
		end
	end
end

Tracking.PLAYER_ENTERING_BATTLEGROUND = Tracking.CacheServerMapping

-- Event handler
local eventFrame = CreateFrame("Frame")
Tracking.eventFrame = eventFrame

eventFrame:SetScript("OnEvent", function(self, event, ...)
	if( Tracking[event] ) then
		Tracking[event](Tracking, ...)
	end

	if( event == "CHAT_MSG_COMBAT_HONOR_GAIN" or event == "PLAYER_PVP_KILLS_CHANGED" or event == "PLAYER_PVP_RANK_CHANGED" or event == "PLAYER_ENTERING_WORLD" ) then
		Tracking:CheckDayChange()
	end
end)

eventFrame:RegisterEvent("CHAT_MSG_COMBAT_HONOR_GAIN")
eventFrame:RegisterEvent("UPDATE_BATTLEFIELD_SCORE")
eventFrame:RegisterEvent("PLAYER_PVP_KILLS_CHANGED")
eventFrame:RegisterEvent("UPDATE_BATTLEFIELD_STATUS")
eventFrame:RegisterEvent("PLAYER_PVP_RANK_CHANGED")
eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
eventFrame:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND")
